@@ -39,7 +39,6 @@
 #include <sys/mman.h>
 #include <signal.h>
 #include <pciaccess.h>
-#include <getopt.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/wait.h>
@@ -52,6 +51,7 @@
 #include "i915_drm.h"
 #include "intel_chipset.h"
 #include "intel_io.h"
+#include "igt_gt.h"
 #include "igt_debugfs.h"
 #include "version.h"
 #include "config.h"
@@ -73,19 +73,41 @@
  * and [batchbuffer](intel-gpu-tools-intel-batchbuffer.html) libraries as dependencies.
  */
 
+uint16_t __drm_device_id;
+
+static int is_i915_device(int fd)
+{
+	drm_version_t version;
+	char name[5] = "";
+
+	memset(&version, 0, sizeof(version));
+	version.name_len = 4;
+	version.name = name;
+
+	if (drmIoctl(fd, DRM_IOCTL_VERSION, &version))
+		return 0;
+
+	return strcmp("i915", name) == 0;
+}
+
 static int
 is_intel(int fd)
 {
 	struct drm_i915_getparam gp;
-	int devid;
+	int devid = 0;
 
+	memset(&gp, 0, sizeof(gp));
 	gp.param = I915_PARAM_CHIPSET_ID;
 	gp.value = &devid;
 
 	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
 		return 0;
 
-	return IS_INTEL(devid);
+	if (!IS_INTEL(devid))
+		return 0;
+
+	__drm_device_id = devid;
+	return 1;
 }
 
 static void check_stop_rings(void)
@@ -194,7 +216,7 @@ int drm_get_card(void)
 		if (fd == -1)
 			continue;
 
-		if (!is_intel(fd)) {
+		if (!is_i915_device(fd) || !is_intel(fd)) {
 			close(fd);
 			continue;
 		}
@@ -209,24 +231,25 @@ int drm_get_card(void)
 }
 
 /** Open the first DRM device we can find, searching up to 16 device nodes */
-static int __drm_open_any(void)
+int __drm_open_any(void)
 {
-	char *name;
-	int ret, fd;
+	for (int i = 0; i < 16; i++) {
+		char name[80];
+		int fd;
 
-	ret = asprintf(&name, "/dev/dri/card%d", drm_get_card());
-	if (ret == -1)
-		return -1;
+		sprintf(name, "/dev/dri/card%u", i);
+		fd = open(name, O_RDWR);
+		if (fd == -1)
+			continue;
 
-	fd = open(name, O_RDWR);
-	free(name);
+		if (is_i915_device(fd) && is_intel(fd))
+			return fd;
 
-	if (!is_intel(fd)) {
 		close(fd);
-		fd = -1;
 	}
 
-	return fd;
+	igt_skip("No intel gpu found\n");
+	return -1;
 }
 
 static int __drm_open_any_render(void)
@@ -246,7 +269,7 @@ static int __drm_open_any_render(void)
 		if (fd == -1)
 			continue;
 
-		if (!is_intel(fd)) {
+		if (!is_i915_device(fd) || !is_intel(fd)) {
 			close(fd);
 			fd = -1;
 			continue;
@@ -286,10 +309,10 @@ static void quiescent_gpu_at_exit_render(int sig)
 /**
  * drm_open_any:
  *
- * Open an i915 drm legacy device node.
+ * Open an i915 drm legacy device node. This function always returns a valid
+ * file descriptor.
  *
- * Returns:
- * The i915 drm file descriptor or -1 on error
+ * Returns: a i915 drm file descriptor
  */
 int drm_open_any(void)
 {
@@ -309,7 +332,26 @@ int drm_open_any(void)
 }
 
 /**
- * drm_open_any:
+ * drm_open_any_master:
+ *
+ * Open an i915 drm legacy device node and ensure that it is drm master.
+ *
+ * Returns:
+ * The i915 drm file descriptor or -1 on error
+ */
+int drm_open_any_master(void)
+{
+	int fd = drm_open_any();
+
+	igt_require(fd >= 0);
+	igt_require_f(drmSetMaster(fd) == 0, "Can't become DRM master, "
+		      "please check if no other DRM client is running.\n");
+
+	return fd;
+}
+
+/**
+ * drm_open_any_render:
  *
  * Open an i915 drm render device node.
  *
