@@ -434,9 +434,8 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
 		break;
 	}
 
-	asprintf(&path, "%s-%d/force",
-		 kmstest_connector_type_str(connector->connector_type),
-		 connector->connector_type_id);
+	igt_assert_neq(asprintf(&path, "%s-%d/force", kmstest_connector_type_str(connector->connector_type), connector->connector_type_id),
+		       -1);
 	debugfs_fd = igt_debugfs_open(path, O_WRONLY | O_TRUNC);
 
 	if (debugfs_fd == -1) {
@@ -494,9 +493,8 @@ void kmstest_force_edid(int drm_fd, drmModeConnector *connector,
 	char *path;
 	int debugfs_fd, ret;
 
-	asprintf(&path, "%s-%d/edid_override",
-		 kmstest_connector_type_str(connector->connector_type),
-		 connector->connector_type_id);
+	igt_assert_neq(asprintf(&path, "%s-%d/edid_override", kmstest_connector_type_str(connector->connector_type), connector->connector_type_id),
+		       -1);
 	debugfs_fd = igt_debugfs_open(path, O_WRONLY | O_TRUNC);
 	free(path);
 
@@ -907,12 +905,14 @@ static void igt_output_refresh(igt_output_t *output)
 	if (!output->valid)
 		return;
 
+	if (output->use_override_mode)
+		output->config.default_mode = output->override_mode;
+
 	if (!output->name) {
 		drmModeConnector *c = output->config.connector;
 
-		asprintf(&output->name, "%s-%d",
-			 kmstest_connector_type_str(c->connector_type),
-			 c->connector_type_id);
+		igt_assert_neq(asprintf(&output->name, "%s-%d", kmstest_connector_type_str(c->connector_type), c->connector_type_id),
+			       -1);
 	}
 
 	LOG(display, "%s: Selecting pipe %s\n", output->name,
@@ -930,16 +930,32 @@ get_plane_property(int drm_fd, uint32_t plane_id, const char *name,
 				    name, prop_id, value, prop);
 }
 
-static void
+static int
 igt_plane_set_property(igt_plane_t *plane, uint32_t prop_id, uint64_t value)
 {
 	igt_pipe_t *pipe = plane->pipe;
 	igt_display_t *display = pipe->display;
 
-	drmModeObjectSetProperty(display->drm_fd, plane->drm_plane->plane_id,
+	return drmModeObjectSetProperty(display->drm_fd, plane->drm_plane->plane_id,
 				 DRM_MODE_OBJECT_PLANE, prop_id, value);
 }
 
+static bool
+get_crtc_property(int drm_fd, uint32_t crtc_id, const char *name,
+		   uint32_t *prop_id /* out */, uint64_t *value /* out */,
+		   drmModePropertyPtr *prop /* out */)
+{
+	return kmstest_get_property(drm_fd, crtc_id, DRM_MODE_OBJECT_CRTC,
+				    name, prop_id, value, prop);
+}
+
+static void
+igt_crtc_set_property(igt_output_t *output, uint32_t prop_id, uint64_t value)
+{
+	drmModeObjectSetProperty(output->display->drm_fd,
+		output->config.crtc->crtc_id, DRM_MODE_OBJECT_CRTC, prop_id, value);
+}
+
 /*
  * Walk a plane's property list to determine its type.  If we don't
  * find a type property, then the kernel doesn't support universal
@@ -1097,6 +1113,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	igt_assert(display->outputs);
 
 	for (i = 0; i < display->n_outputs; i++) {
+		int j;
 		igt_output_t *output = &display->outputs[i];
 
 		/*
@@ -1108,6 +1125,19 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		output->display = display;
 
 		igt_output_refresh(output);
+
+		for (j = 0; j < display->n_pipes; j++) {
+			uint64_t prop_value;
+			igt_pipe_t *pipe = &display->pipes[j];
+			if (output->config.crtc) {
+				get_crtc_property(display->drm_fd, output->config.crtc->crtc_id,
+						   "background_color",
+						   &pipe->background_property,
+						   &prop_value,
+						   NULL);
+				pipe->background = (uint32_t)prop_value;
+			}
+		}
 	}
 
 	drmModeFreePlaneResources(plane_resources);
@@ -1283,6 +1313,14 @@ static int igt_drm_plane_commit(igt_plane_t *plane,
 	igt_display_t *display = output->display;
 	uint32_t fb_id, crtc_id;
 	int ret;
+	uint32_t src_x;
+	uint32_t src_y;
+	uint32_t src_w;
+	uint32_t src_h;
+	int32_t crtc_x;
+	int32_t crtc_y;
+	uint32_t crtc_w;
+	uint32_t crtc_h;
 
 	igt_assert(plane->drm_plane);
 
@@ -1293,7 +1331,7 @@ static int igt_drm_plane_commit(igt_plane_t *plane,
 	fb_id = igt_plane_get_fb_id(plane);
 	crtc_id = output->config.crtc->crtc_id;
 
-	if (plane->fb_changed && fb_id == 0) {
+	if ((plane->fb_changed || plane->size_changed) && fb_id == 0) {
 		LOG(display,
 		    "%s: SetPlane pipe %s, plane %d, disabling\n",
 		    igt_output_name(output),
@@ -1313,38 +1351,50 @@ static int igt_drm_plane_commit(igt_plane_t *plane,
 				      IGT_FIXED(0,0) /* src_h */);
 
 		CHECK_RETURN(ret, fail_on_error);
-	} else if (plane->fb_changed || plane->position_changed) {
+	} else if (plane->fb_changed || plane->position_changed ||
+		plane->size_changed) {
+		src_x = IGT_FIXED(plane->fb->src_x,0); /* src_x */
+		src_y = IGT_FIXED(plane->fb->src_y,0); /* src_y */
+		src_w = IGT_FIXED(plane->fb->src_w,0); /* src_w */
+		src_h = IGT_FIXED(plane->fb->src_h,0); /* src_h */
+		crtc_x = plane->crtc_x;
+		crtc_y = plane->crtc_y;
+		crtc_w = plane->crtc_w;
+		crtc_h = plane->crtc_h;
+
 		LOG(display,
-		    "%s: SetPlane %s.%d, fb %u, position (%d, %d)\n",
+		    "%s: SetPlane %s.%d, fb %u, src = (%d, %d) "
+			"%ux%u dst = (%u, %u) %ux%u\n",
 		    igt_output_name(output),
 		    kmstest_pipe_name(output->config.pipe),
 		    plane->index,
 		    fb_id,
-		    plane->crtc_x, plane->crtc_y);
+		    src_x >> 16, src_y >> 16, src_w >> 16, src_h >> 16,
+		    crtc_x, crtc_y, crtc_w, crtc_h);
 
 		ret = drmModeSetPlane(display->drm_fd,
 				      plane->drm_plane->plane_id,
 				      crtc_id,
 				      fb_id,
 				      0,    /* flags */
-				      plane->crtc_x, plane->crtc_y,
-				      plane->crtc_w, plane->crtc_h,
-				      IGT_FIXED(0,0), /* src_x */
-				      IGT_FIXED(0,0), /* src_y */
-				      IGT_FIXED(plane->fb->width,0), /* src_w */
-				      IGT_FIXED(plane->fb->height,0) /* src_h */);
+				      crtc_x, crtc_y,
+				      crtc_w, crtc_h,
+				      src_x, src_y,
+				      src_w, src_h);
 
 		CHECK_RETURN(ret, fail_on_error);
 	}
 
 	plane->fb_changed = false;
 	plane->position_changed = false;
+	plane->size_changed = false;
 
 	if (plane->rotation_changed) {
-		igt_plane_set_property(plane, plane->rotation_property,
+		ret = igt_plane_set_property(plane, plane->rotation_property,
 				       plane->rotation);
 
 		plane->rotation_changed = false;
+		CHECK_RETURN(ret, fail_on_error);
 	}
 
 	return 0;
@@ -1433,7 +1483,7 @@ static int igt_primary_plane_commit_legacy(igt_plane_t *primary,
 	igt_assert(!primary->rotation_changed);
 
 	if (!primary->fb_changed && !primary->position_changed &&
-	    !primary->panning_changed)
+		!primary->size_changed && !primary->panning_changed)
 		return 0;
 
 	crtc_id = output->config.crtc->crtc_id;
@@ -1480,6 +1530,7 @@ static int igt_primary_plane_commit_legacy(igt_plane_t *primary,
 	primary->pipe->enabled = (fb_id != 0);
 	primary->fb_changed = false;
 	primary->position_changed = false;
+	primary->size_changed = false;
 	primary->panning_changed = false;
 
 	return 0;
@@ -1527,10 +1578,17 @@ static int igt_output_commit(igt_output_t *output,
 
 	pipe = igt_output_get_driving_pipe(output);
 
+	if (pipe->background_changed) {
+		igt_crtc_set_property(output, pipe->background_property,
+			pipe->background);
+
+		pipe->background_changed = false;
+	}
+
 	for (i = 0; i < pipe->n_planes; i++) {
 		igt_plane_t *plane = &pipe->planes[i];
 
-		if (plane->fb_changed || plane->position_changed)
+		if (plane->fb_changed || plane->position_changed || plane->size_changed)
 			need_wait_for_vblank = true;
 
 		ret = igt_plane_commit(plane, output, s, fail_on_error);
@@ -1659,6 +1717,21 @@ drmModeModeInfo *igt_output_get_mode(igt_output_t *output)
 	return &output->config.default_mode;
 }
 
+/**
+ * igt_output_override_mode:
+ * @output: Output of which the mode will be overriden
+ * @mode: New mode
+ *
+ * Overrides the output's mode with @mode, so that it is used instead of the
+ * mode obtained with get connectors. Note that the mode is used without
+ * checking if the output supports it, so this might lead to unexpect results.
+ */
+void igt_output_override_mode(igt_output_t *output, drmModeModeInfo *mode)
+{
+	output->override_mode = *mode;
+	output->use_override_mode = true;
+}
+
 void igt_output_set_pipe(igt_output_t *output, enum pipe pipe)
 {
 	igt_display_t *display = output->display;
@@ -1692,14 +1765,22 @@ void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb)
 	plane->fb = fb;
 	/* hack to keep tests working that don't call igt_plane_set_size() */
 	if (fb) {
+		/* set default plane size as fb size */
 		plane->crtc_w = fb->width;
 		plane->crtc_h = fb->height;
+
+		/* set default src pos/size as fb size */
+		fb->src_x = 0;
+		fb->src_y = 0;
+		fb->src_w = fb->width;
+		fb->src_h = fb->height;
 	} else {
 		plane->crtc_w = 0;
 		plane->crtc_h = 0;
 	}
 
 	plane->fb_changed = true;
+	plane->size_changed = true;
 }
 
 void igt_plane_set_position(igt_plane_t *plane, int x, int y)
@@ -1716,21 +1797,78 @@ void igt_plane_set_position(igt_plane_t *plane, int x, int y)
 	plane->position_changed = true;
 }
 
+/**
+ * igt_plane_set_size:
+ * @plane: plane pointer for which size to be set
+ * @w: width
+ * @h: height
+ *
+ * This function sets width and height for requested plane.
+ * New size will be commited at plane commit time via
+ * drmModeSetPlane().
+ */
 void igt_plane_set_size(igt_plane_t *plane, int w, int h)
 {
 	igt_pipe_t *pipe = plane->pipe;
 	igt_display_t *display = pipe->display;
 
-	LOG(display, "%s.%d: plane_set_size(%d,%d)\n",
+	LOG(display, "%s.%d: plane_set_size (%dx%d)\n",
 	    kmstest_pipe_name(pipe->pipe), plane->index, w, h);
 
 	plane->crtc_w = w;
 	plane->crtc_h = h;
 
-	/*
-	 * must be fb_changed so that legacy cursors call
-	 * drmModeSetCursor() instead of drmModeMoveCursor()
-	 */
+	plane->size_changed = true;
+}
+
+/**
+ * igt_fb_set_position:
+ * @fb: framebuffer pointer
+ * @plane: plane
+ * @x: X position
+ * @y: Y position
+ *
+ * This function sets position for requested framebuffer as src to plane.
+ * New position will be committed at plane commit time via drmModeSetPlane().
+ */
+void igt_fb_set_position(struct igt_fb *fb, igt_plane_t *plane,
+	uint32_t x, uint32_t y)
+{
+	igt_pipe_t *pipe = plane->pipe;
+	igt_display_t *display = pipe->display;
+
+	LOG(display, "%s.%d: fb_set_position(%d,%d)\n",
+	    kmstest_pipe_name(pipe->pipe), plane->index, x, y);
+
+	fb->src_x = x;
+	fb->src_y = y;
+
+	plane->fb_changed = true;
+}
+
+/**
+ * igt_fb_set_size:
+ * @fb: framebuffer pointer
+ * @plane: plane
+ * @w: width
+ * @h: height
+ *
+ * This function sets fetch rect size from requested framebuffer as src
+ * to plane. New size will be committed at plane commit time via
+ * drmModeSetPlane().
+ */
+void igt_fb_set_size(struct igt_fb *fb, igt_plane_t *plane,
+	uint32_t w, uint32_t h)
+{
+	igt_pipe_t *pipe = plane->pipe;
+	igt_display_t *display = pipe->display;
+
+	LOG(display, "%s.%d: fb_set_position(%dx%d)\n",
+	    kmstest_pipe_name(pipe->pipe), plane->index, w, h);
+
+	fb->src_w = w;
+	fb->src_h = h;
+
 	plane->fb_changed = true;
 }
 
@@ -1779,6 +1917,30 @@ void igt_plane_set_rotation(igt_plane_t *plane, igt_rotation_t rotation)
 	plane->rotation_changed = true;
 }
 
+/**
+ * igt_crtc_set_background:
+ * @pipe: pipe pointer to which background color to be set
+ * @background: background color value in BGR 16bpc
+ *
+ * Sets background color for requested pipe. Color value provided here
+ * will be actually submitted at output commit time via "background_color"
+ * property.
+ * For example to get red as background, set background = 0x00000000FFFF.
+ */
+void igt_crtc_set_background(igt_pipe_t *pipe, uint64_t background)
+{
+	igt_display_t *display = pipe->display;
+
+	LOG(display, "%s.%d: crtc_set_background(%lu)\n",
+	    kmstest_pipe_name(pipe->pipe),
+	    pipe->pipe, background);
+
+	pipe->background = background;
+
+	pipe->background_changed = true;
+}
+
+
 void igt_wait_for_vblank(int drm_fd, enum pipe pipe)
 {
 	drmVBlank wait_vbl;
