@@ -40,6 +40,7 @@
 #include "rendercopy.h"
 #include "media_fill.h"
 #include "ioctl_wrappers.h"
+#include "media_spin.h"
 
 #include <i915_drm.h>
 
@@ -333,15 +334,6 @@ intel_blt_copy(struct intel_batchbuffer *batch,
 	uint32_t cmd_bits = 0;
 	uint32_t br13_bits;
 
-#define CHECK_RANGE(x)	((x) >= 0 && (x) < (1 << 15))
-	igt_assert(CHECK_RANGE(src_x1) && CHECK_RANGE(src_y1) &&
-		   CHECK_RANGE(dst_x1) && CHECK_RANGE(dst_y1) &&
-		   CHECK_RANGE(width) && CHECK_RANGE(height) &&
-		   CHECK_RANGE(src_x1 + width) && CHECK_RANGE(src_y1 + height)
-		   && CHECK_RANGE(dst_x1 + width) && CHECK_RANGE(dst_y1 +
-								 height) &&
-		   CHECK_RANGE(src_pitch) && CHECK_RANGE(dst_pitch));
-#undef CHECK_RANGE
 	igt_assert(bpp*(src_x1 + width) <= 8*src_pitch);
 	igt_assert(bpp*(dst_x1 + width) <= 8*dst_pitch);
 	igt_assert(src_pitch * (src_y1 + height) <= src_bo->size);
@@ -360,6 +352,18 @@ intel_blt_copy(struct intel_batchbuffer *batch,
 		cmd_bits |= XY_SRC_COPY_BLT_DST_TILED;
 	}
 
+#define CHECK_RANGE(x)	((x) >= 0 && (x) < (1 << 15))
+	igt_assert(CHECK_RANGE(src_x1) && CHECK_RANGE(src_y1) &&
+		   CHECK_RANGE(dst_x1) && CHECK_RANGE(dst_y1) &&
+		   CHECK_RANGE(width) && CHECK_RANGE(height) &&
+		   CHECK_RANGE(src_x1 + width) &&
+		   CHECK_RANGE(src_y1 + height) &&
+		   CHECK_RANGE(dst_x1 + width) &&
+		   CHECK_RANGE(dst_y1 + height) &&
+		   CHECK_RANGE(src_pitch) &&
+		   CHECK_RANGE(dst_pitch));
+#undef CHECK_RANGE
+
 	br13_bits = 0;
 	switch (bpp) {
 	case 8:
@@ -373,7 +377,7 @@ intel_blt_copy(struct intel_batchbuffer *batch,
 			    XY_SRC_COPY_BLT_WRITE_RGB;
 		break;
 	default:
-		igt_fail(1);
+		igt_fail(IGT_EXIT_FAILURE);
 	}
 
 	BLIT_COPY_BATCH_START(cmd_bits);
@@ -548,7 +552,7 @@ fill_object(struct drm_i915_gem_exec_object2 *obj, uint32_t gem_handle,
 	memset(obj, 0, sizeof(*obj));
 	obj->handle = gem_handle;
 	obj->relocation_count = count;
-	obj->relocs_ptr = (uint64_t)relocs;
+	obj->relocs_ptr = (uintptr_t)relocs;
 }
 
 static void exec_blit(int fd,
@@ -557,7 +561,7 @@ static void exec_blit(int fd,
 {
 	struct drm_i915_gem_execbuffer2 exec;
 
-	exec.buffers_ptr = (uint64_t)objs;
+	exec.buffers_ptr = (uintptr_t)objs;
 	exec.buffer_count = count;
 	exec.batch_start_offset = 0;
 	exec.batch_len = batch_len * 4;
@@ -785,3 +789,26 @@ igt_fillfunc_t igt_get_gpgpu_fillfunc(int devid)
 
 	return fill;
 }
+
+/**
+ * igt_get_media_spinfunc:
+ * @devid: pci device id
+ *
+ * Returns:
+ *
+ * The platform-specific media spin function pointer for the device specified
+ * with @devid. Will return NULL when no media spin function is implemented.
+ */
+igt_media_spinfunc_t igt_get_media_spinfunc(int devid)
+{
+	igt_media_spinfunc_t spin = NULL;
+
+	if (IS_GEN9(devid))
+		spin = gen9_media_spinfunc;
+	else if (IS_BROADWELL(devid))
+		spin = gen8_media_spinfunc;
+	else if (IS_CHERRYVIEW(devid))
+		spin = gen8lp_media_spinfunc;
+
+	return spin;
+}
