@@ -41,7 +41,7 @@
 #include <sys/ioctl.h>
 #include <pthread.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 
 #define OBJECT_SIZE (128*1024) /* restricted to 1MiB alignment on i915 fences */
@@ -68,7 +68,7 @@ bo_create (int fd, int tiling)
 	handle = gem_create(fd, OBJECT_SIZE);
 
 	/* dirty cpu caches a bit ... */
-	ptr = gem_mmap__cpu(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
+	ptr = gem_mmap__cpu(fd, handle, 0, OBJECT_SIZE, PROT_READ | PROT_WRITE);
 	igt_assert(ptr);
 	memset(ptr, 0, OBJECT_SIZE);
 	munmap(ptr, OBJECT_SIZE);
@@ -113,8 +113,8 @@ _bo_write_verify(struct test *t)
 	unsigned int dwords = OBJECT_SIZE >> 2;
 	const char *tile_str[] = { "none", "x", "y" };
 
-	assert (t->tiling >= 0 && t->tiling <= I915_TILING_Y);
-	assert (t->num_surfaces > 0);
+	igt_assert(t->tiling >= 0 && t->tiling <= I915_TILING_Y);
+	igt_assert_lt(0, t->num_surfaces);
 
 	s = calloc(sizeof(*s), t->num_surfaces);
 	igt_assert(s);
@@ -171,17 +171,17 @@ static int run_test(int threads_per_fence, void *f, int tiling,
 	t.num_surfaces = surfaces_per_thread;
 
 	num_fences = gem_available_fences(t.fd);
-	assert (num_fences > 0);
+	igt_assert_lt(0, num_fences);
 
 	num_threads = threads_per_fence * num_fences;
 
-	printf("%s: threads %d, fences %d, tiling %d, surfaces per thread %d\n",
-	       f == bo_copy ? "copy" : "write-verify", num_threads,
-	       num_fences, tiling, surfaces_per_thread);
+	igt_info("%s: threads %d, fences %d, tiling %d, surfaces per thread %d\n",
+		 f == bo_copy ? "copy" : "write-verify", num_threads,
+		 num_fences, tiling, surfaces_per_thread);
 
 	if (threads_per_fence) {
 		threads = calloc(sizeof(*threads), num_threads);
-		assert  (threads != NULL);
+		igt_assert(threads != NULL);
 
 		for (n = 0; n < num_threads; n++)
 			pthread_create (&threads[n], NULL, f, &t);
@@ -203,29 +203,29 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_subtest("bo-write-verify-none")
-		assert (run_test(0, bo_write_verify, I915_TILING_NONE, 80) == 0);
+		igt_assert(run_test(0, bo_write_verify, I915_TILING_NONE, 80) == 0);
 
 	igt_subtest("bo-write-verify-x")
-		assert (run_test(0, bo_write_verify, I915_TILING_X, 80) == 0);
+		igt_assert(run_test(0, bo_write_verify, I915_TILING_X, 80) == 0);
 
 	igt_subtest("bo-write-verify-y")
-		assert (run_test(0, bo_write_verify, I915_TILING_Y, 80) == 0);
+		igt_assert(run_test(0, bo_write_verify, I915_TILING_Y, 80) == 0);
 
 	igt_subtest("bo-write-verify-threaded-none")
-		assert (run_test(5, bo_write_verify, I915_TILING_NONE, 2) == 0);
+		igt_assert(run_test(5, bo_write_verify, I915_TILING_NONE, 2) == 0);
 
 	igt_subtest("bo-write-verify-threaded-x") {
-		assert (run_test(2, bo_write_verify, I915_TILING_X, 2) == 0);
-		assert (run_test(5, bo_write_verify, I915_TILING_X, 2) == 0);
-		assert (run_test(10, bo_write_verify, I915_TILING_X, 2) == 0);
-		assert (run_test(20, bo_write_verify, I915_TILING_X, 2) == 0);
+		igt_assert(run_test(2, bo_write_verify, I915_TILING_X, 2) == 0);
+		igt_assert(run_test(5, bo_write_verify, I915_TILING_X, 2) == 0);
+		igt_assert(run_test(10, bo_write_verify, I915_TILING_X, 2) == 0);
+		igt_assert(run_test(20, bo_write_verify, I915_TILING_X, 2) == 0);
 	}
 
 	igt_subtest("bo-write-verify-threaded-y") {
-		assert (run_test(2, bo_write_verify, I915_TILING_Y, 2) == 0);
-		assert (run_test(5, bo_write_verify, I915_TILING_Y, 2) == 0);
-		assert (run_test(10, bo_write_verify, I915_TILING_Y, 2) == 0);
-		assert (run_test(20, bo_write_verify, I915_TILING_Y, 2) == 0);
+		igt_assert(run_test(2, bo_write_verify, I915_TILING_Y, 2) == 0);
+		igt_assert(run_test(5, bo_write_verify, I915_TILING_Y, 2) == 0);
+		igt_assert(run_test(10, bo_write_verify, I915_TILING_Y, 2) == 0);
+		igt_assert(run_test(20, bo_write_verify, I915_TILING_Y, 2) == 0);
 	}
 
 	igt_subtest("bo-copy")
