@@ -41,12 +41,13 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
-#include "drm.h"
-#include "i915_drm.h"
-#include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
-#include "rendercopy.h"
+#include <drm.h>
+
+#include "ioctl_wrappers.h"
+#include "drmtest.h"
+#include "intel_chipset.h"
+#include "intel_io.h"
+#include "igt_aux.h"
 
 struct bo {
 	const char *ring;
@@ -94,8 +95,8 @@ static int check_bo(struct bo *b)
 			int x = i % width;
 			int y = i / width;
 
-			printf("%s: copy #%d at %d,%d failed: read 0x%08x\n",
-			       b->ring, i, x, y, map[i]);
+			igt_info("%s: copy #%d at %d,%d failed: read 0x%08x\n",
+				 b->ring, i, x, y, map[i]);
 		}
 	}
 	drm_intel_bo_unmap(b->dst);
@@ -113,9 +114,9 @@ static void destroy_bo(struct bo *b)
 static int check_ring(drm_intel_bufmgr *bufmgr,
 		      struct intel_batchbuffer *batch,
 		      const char *ring,
-		      render_copyfunc_t copy)
+		      igt_render_copyfunc_t copy)
 {
-	struct scratch_buf src, tmp, dst;
+	struct igt_buf src, tmp, dst;
 	struct bo bo;
 	char output[100];
 	int i;
@@ -126,7 +127,6 @@ static int check_ring(drm_intel_bufmgr *bufmgr,
 
 	src.stride = 4 * width;
 	src.tiling = 0;
-	src.data = src.cpu_mapping = NULL;
 	src.size = 4 * width * height;
 	src.num_tiles = 4 * width * height;
 	dst = tmp = src;
@@ -156,7 +156,7 @@ static int check_ring(drm_intel_bufmgr *bufmgr,
 
 		igt_progress(output, i, width*height);
 
-		igt_assert(y < height);
+		igt_assert_lt(y, height);
 
 		/* Dummy load to fill the ring */
 		copy(batch, NULL, &src, 0, 0, width, height, &tmp, 0, 0);
@@ -165,7 +165,7 @@ static int check_ring(drm_intel_bufmgr *bufmgr,
 	}
 
 	/* verify */
-	printf("verifying\n");
+	igt_info("verifying\n");
 	i = check_bo(&bo);
 	destroy_bo(&bo);
 
@@ -174,22 +174,20 @@ static int check_ring(drm_intel_bufmgr *bufmgr,
 
 static void blt_copy(struct intel_batchbuffer *batch,
 		     drm_intel_context *context,
-		     struct scratch_buf *src, unsigned src_x, unsigned src_y,
+		     struct igt_buf *src, unsigned src_x, unsigned src_y,
 		     unsigned w, unsigned h,
-		     struct scratch_buf *dst, unsigned dst_x, unsigned dst_y)
+		     struct igt_buf *dst, unsigned dst_x, unsigned dst_y)
 {
-	BLIT_COPY_BATCH_START(batch->devid, 0);
+	BLIT_COPY_BATCH_START(0);
 	OUT_BATCH((3 << 24) | /* 32 bits */
 		  (0xcc << 16) | /* copy ROP */
 		  dst->stride);
 	OUT_BATCH((dst_y << 16) | dst_x); /* dst x1,y1 */
 	OUT_BATCH(((dst_y + h) << 16) | (dst_x + w)); /* dst x2,y2 */
-	OUT_RELOC(dst->bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-	BLIT_RELOC_UDW(batch->devid);
+	OUT_RELOC_FENCED(dst->bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
 	OUT_BATCH((src_y << 16) | src_x); /* src x1,y1 */
 	OUT_BATCH(src->stride);
-	OUT_RELOC(src->bo, I915_GEM_DOMAIN_RENDER, 0, 0);
-	BLIT_RELOC_UDW(batch->devid);
+	OUT_RELOC_FENCED(src->bo, I915_GEM_DOMAIN_RENDER, 0, 0);
 	ADVANCE_BATCH();
 
 	intel_batchbuffer_flush(batch);
@@ -197,7 +195,6 @@ static void blt_copy(struct intel_batchbuffer *batch,
 
 drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
-render_copyfunc_t copy;
 int fd;
 
 igt_main
@@ -215,16 +212,34 @@ igt_main
 	igt_subtest("blitter")
 		check_ring(bufmgr, batch, "blt", blt_copy);
 
-	igt_fixture {
-		/* Strictly only required on architectures with a separate BLT ring,
-		 * but lets stress everybody.
-		 */
-		copy = get_render_copyfunc(batch->devid);
+	/* Strictly only required on architectures with a separate BLT ring,
+	 * but lets stress everybody.
+	 */
+	igt_subtest("render") {
+		igt_render_copyfunc_t copy;
+
+		copy = igt_get_render_copyfunc(batch->devid);
 		igt_require(copy);
+
+		check_ring(bufmgr, batch, "render", copy);
 	}
 
-	igt_subtest("render")
+	igt_fork_signal_helper();
+	igt_subtest("blitter-interruptible")
+		check_ring(bufmgr, batch, "blt", blt_copy);
+
+	/* Strictly only required on architectures with a separate BLT ring,
+	 * but lets stress everybody.
+	 */
+	igt_subtest("render-interruptible") {
+		igt_render_copyfunc_t copy;
+
+		copy = igt_get_render_copyfunc(batch->devid);
+		igt_require(copy);
+
 		check_ring(bufmgr, batch, "render", copy);
+	}
+	igt_stop_signal_helper();
 
 	igt_fixture {
 		intel_batchbuffer_free(batch);
