@@ -319,7 +319,7 @@ render_copy(int fd,
 		drmCommandNone(fd, DRM_I915_GEM_THROTTLE);
 		ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	}
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	gem_close(fd, handle);
 }
@@ -397,7 +397,7 @@ static void blt_copy(int fd, uint32_t dst, uint32_t src)
 		drmCommandNone(fd, DRM_I915_GEM_THROTTLE);
 		ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	}
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	gem_close(fd, handle);
 }
@@ -463,7 +463,7 @@ int main(int argc, char **argv)
 	uint32_t start = 0;
 	int i, fd, count;
 
-	igt_simple_init();
+	igt_simple_init(argc, argv);
 
 	fd = drm_open_any();
 
@@ -533,5 +533,5 @@ int main(int argc, char **argv)
 		check_bo(fd, handle[i], start_val[i]);
 	igt_info("done\n");
 
-	return 0;
+	igt_exit();
 }
